from kuto.core.android.driver import AndroidDriver
from kuto.core.android.element import AdrElem
from kuto.core.ios.driver import IosDriver
from kuto.core.ios.element import IosElem
from kuto.core.image.element import ImageElem
from kuto.core.ocr.element import OCRElem
from kuto.core.web.driver import PlayWrightDriver as WebDriver
from kuto.core.web.element import WebElem, FraElem
from kuto.core.api.request import HttpReq
from requests_toolbelt import MultipartEncoder

__all__ = [
    "AndroidDriver",
    "AdrElem",
    "IosDriver",
    "IosElem",
    "ImageElem",
    "OCRElem",
    "WebDriver",
    "WebElem",
    "FraElem",
    "HttpReq",
    "MultipartEncoder"
]




"""
@Author: kang.yang
@Date: 2023/8/2 20:23
"""
from kuto import Page, WebElem as Elem

"""
定位方式：优先使用css，大规模开展自动化前需要开发给相关元素加上唯一标识的测试id
css: 根据cssSelector进行定位，https://zhuanlan.zhihu.com/p/571510714
xpath: 根据xpath进行定位，教程：https://zhuanlan.zhihu.com/p/571060826
text: 根据元素的可视文本内容进行定位
holder: 根据输入框placeHolder进行定位
index: 获取第index个定位到的元素
"""


class IndexPage(Page):
    """首页"""
    loginBtn = Elem(text='登录/注册')
    patentText = Elem(text='查专利')


class LoginPage(Page):
    """登录页"""
    pwdLoginTab = Elem(text='帐号密码登录')
    userInput = Elem(holder='请输入手机号码')
    pwdInput = Elem(holder='请输入密码')
    licenseBtn = Elem(css="span.el-checkbox__inner", index=1)
    loginBtn = Elem(text='立即登录')
    firstCompanyIcon = Elem(xpath="(//img[@class='right-icon'])[1]")




import typing

from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from kuto.core.android.driver import AndroidDriver
from kuto.utils.exceptions import NoSuchElementException
from kuto.utils.log import logger
from kuto.utils.common import dict_to_str, calculate_time


class AdrElem(object):
    """
    安卓元素定义
    """

    def __init__(self,
                 AdrDriver: AndroidDriver = None,
                 rid: str = None,
                 className: str = None,
                 text: str = None,
                 textCont: str = None,
                 xpath: str = None,
                 index: int = None):
        """
        @param AdrDriver: 安卓驱动，必填
        @param rid: resourceId定位
        @param className: className定位
        @param text: text定位
        @param textCont: text包含
        @param xpath: xpath定位
        @param index: 定位出多个元素时，指定索引，从0开始
        """
        self._driver = AdrDriver

        self._kwargs = {}
        if rid is not None:
            self._kwargs["resourceId"] = rid
        if className is not None:
            self._kwargs["className"] = className
        if text is not None:
            self._kwargs["text"] = text
        if textCont is not None:
            self._kwargs["textContains"] = textCont
        if xpath:
            self._kwargs["xpath"] = xpath
        if index is not None:
            self._kwargs["instance"] = index

        self._xpath = xpath

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    @calculate_time
    def find(self, timeout=5):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @return:
        """
        if self._xpath is not None:
            logger.info(f'查找控件: xpath={self._xpath}')
        else:
            logger.info(f'查找控件: {self._kwargs}')
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath is not None else self._driver.d(**self._kwargs)

        if _element.wait(timeout=timeout):
            logger.info(f"查找成功")
            return _element
        else:
            logger.info(f"查找失败")
            self._driver.screenshot(dict_to_str(self._kwargs) + "_查找失败")
            raise NoSuchElementException(f"控件 {self._kwargs} 查找失败")

    @property
    def text(self):
        logger.info("获取控件文本属性")
        _elem = self.find(timeout=3)
        if isinstance(_elem, XPathSelector):
            elems = _elem.all()
        else:
            elems = list(_elem)
        text = []
        for elem in elems:
            text.append(elem.get_text())
        logger.info(text)
        return text

    def exists(self, timeout=3):
        logger.info("检查控件是否存在")
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath is not None else self._driver.d(**self._kwargs)
        result = True if _element.wait(timeout=timeout) is not None else False
        logger.info(result)
        return result

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector],
                      offset=(0.5, 0.5)):
        """
        修正控件中心坐标
        """
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    def click(self, timeout=5):
        logger.info("点击控件")
        element = self.find(timeout=timeout)
        # 这种方式经常点击不成功，感觉是页面刷新有影响
        # element.click()
        x, y = self._adapt_center(element)
        self._driver.d.click(x, y)

    def click_exists(self, timeout=3):
        logger.info("控件存在才点击")
        if self.exists(timeout=timeout):
            self.click()

    def input(self, text):
        logger.info(f"输入文本: {text}")
        self.find().set_text(text)

    def input_exists(self, text: str, timeout=3):
        logger.info(f"控件存在才输入: {text}")
        if self.exists(timeout=timeout):
            self.input(text)

    def input_pwd(self, text):
        """密码输入框输入有时候用input输入不了"""
        logger.info(f"输入密码: {text}")
        self.find().click()
        self._driver.d(focused=True).set_text(text)

    def clear(self):
        logger.info("清空输入框")
        self.find().clear_text()

    def assert_exists(self, timeout=3):
        logger.info("断言控件存在")
        status = self.exists(timeout=timeout)
        assert status, "控件不存在"

    def assert_text(self, text, timeout=3):
        logger.info(f"断言控件文本属性包括: {text}")
        self.find(timeout=timeout)
        _text = self.text
        assert text in _text, f"文本属性 {_text} 不包含 {text}"


if __name__ == '__main__':
    driver = AndroidDriver(
        device_id="UJK0220521066836",
        pkg_name="com.qizhidao.clientapp"
    )
    print(AdrElem(driver, textCont="华为").text)







import time

from kuto.utils.log import logger
from kuto.core.ios.driver import IosDriver
from kuto.utils.common import dict_to_str, calculate_time
from kuto.utils.exceptions import NoSuchElementException


class IosElem(object):
    """
    IOS原生元素定义
    """

    def __init__(self,
                 iosDriver: IosDriver = None,
                 name: str = None,
                 label: str = None,
                 labelCont: str = None,
                 value: str = None,
                 valueCont: str = None,
                 text: str = None,
                 textCont: str = None,
                 className: str = None,
                 xpath: str = None,
                 index: int = None):
        """
        param iosDriver,
        param name,
        param label,
        param value,
        param text,
        param className,
        param xpath,
        param index: 索引
        """
        self._driver = iosDriver

        self._kwargs = {}
        if name is not None:
            self._kwargs["name"] = name
        if label is not None:
            self._kwargs["label"] = label
        if labelCont is not None:
            self._kwargs["labelContains"] = labelCont
        if value is not None:
            self._kwargs["value"] = value
        if valueCont is not None:
            self._kwargs["valueContains"] = valueCont
        if text is not None:
            self._kwargs["text"] = text
        if textCont is not None:
            self._kwargs["textContains"] = textCont
        if className is not None:
            self._kwargs["className"] = className
        if index is not None:
            self._kwargs["index"] = index

        self._xpath = xpath

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    @calculate_time
    def find(self, timeout=5):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param timeout:
        @return:
        """
        if self._xpath is not None:
            logger.info(f'查找控件: xpath={self._xpath}')
        else:
            logger.info(f'查找控件: {self._kwargs}')

        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath else self._driver.d(**self._kwargs)
        try:
            if _element.wait(timeout=timeout):
                logger.info(f"查找成功")
                return _element
            else:
                logger.info(f"查找失败")
                self._driver.screenshot(dict_to_str(self._kwargs) + "_查找失败")
                raise NoSuchElementException(f"控件 {self._kwargs} 查找失败")
        except ConnectionError:
            logger.info('wda连接失败, 进行重连!!!')
            # 由于WDA会意外链接错误
            self._driver = IosDriver(self._driver.device_id, self._driver.pkg_name)
            time.sleep(5)

            logger.info('重连成功, 重新开始查找控件')
            if _element.wait(timeout=timeout):
                logger.info(f"查找成功")
                return _element
            else:
                logger.info(f"查找失败")
                self._driver.screenshot(dict_to_str(self._kwargs))
                raise NoSuchElementException(f"控件 {self._kwargs} 查找失败")

    @property
    def text(self):
        """获取元素文本"""
        logger.info(f"获取空间文本属性")
        text = [elem.text for elem in self.find(timeout=5).find_elements()]
        logger.info(text)
        return text

    def exists(self, timeout=3):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info("检查控件是否存在")
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath else self._driver.d(**self._kwargs)
        result = _element.wait(timeout=timeout, raise_error=False)
        logger.info(result)
        return result

    def _adapt_center(self, timeout=5):
        """
        修正控件中心坐标
        """
        bounds = self.find(timeout=timeout).bounds
        left_top_x, left_top_y, width, height = \
            bounds.x, bounds.y, bounds.width, bounds.height
        center_x = int(left_top_x + width/2)
        center_y = int(left_top_y + height/2)
        logger.info(f'{center_x}, {center_y}')
        return center_x, center_y

    def click(self, timeout=5):
        """
        单击
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        """
        logger.info('点击控件')
        x, y = self._adapt_center(timeout=timeout)
        self._driver.d.appium_settings({"snapshotMaxDepth": 0})
        self._driver.d.tap(x, y)
        self._driver.d.appium_settings({"snapshotMaxDepth": 50})

    def click_exists(self, timeout=3):
        logger.info(f"控件存在才点击")
        if self.exists(timeout=timeout):
            self.click()

    def clear(self):
        """清除文本"""
        logger.info("清除输入框文本")
        self.find().clear_text()

    def input(self, text):
        """输入内容"""
        logger.info(f"输入文本：{text}")
        self.find().set_text(text)

    def input_exists(self, text: str, timeout=3):
        logger.info(f"控件存在才输入: {text}")
        if self.exists(timeout=timeout):
            self.input(text)
            logger.info("输入成功")
        else:
            logger.info("控件不存在")

    def assert_exists(self, timeout=3):
        logger.info("断言控件存在")
        status = self.exists(timeout=timeout)
        assert status, f"控件不存在"

    def assert_text(self, text, timeout=3):
        logger.info(f"断言控件文本属性包括: {text}")
        self.find(timeout=timeout)
        _text = self.text
        assert text in _text, f"文本属性 {_text} 不包含 {text}"


if __name__ == '__main__':
    driver = IosDriver(
        device_id="00008101-000E646A3C29003A",
        pkg_name="com.qizhidao.company"
    )
    print(IosElem(driver, labelCont="华为").text)


















"""
@Author: kang.yang
@Date: 2023/8/21 17:05
"""
import kuto

"""
ocr识别可以配合安卓应用或者IOS应用进行使用
"""


class OcrPage(kuto.Page):
    searchBtn = kuto.IosElem(text="搜索", cname="XCUIElementTypeSearchField")
    searchInput = kuto.IosElem(cname="XCUIElementTypeSearchField")
    searchResult = kuto.IosElem(xpath="//Table/Cell[2]")
    schoolEntry = kuto.OCRElem(text="校园场馆", pos=12)



"""
@Author: kang.yang
@Date: 2023/8/21 17:10
"""
import kuto

"""
图像识别可以配合安卓应用或者IOS应用使用
"""


class ImagePage(kuto.Page):
    searchEntry = kuto.AdrElem(rid="com.tencent.mm:id/j5t")
    searchInput = kuto.AdrElem(rid="com.tencent.mm:id/cd7")
    searchResult = kuto.AdrElem(rid="com.tencent.mm:id/kpm")
    schoolEntry = kuto.ImageElem(image="../static/校园场馆.png")



"""
@Author: kang.yang
@Date: 2023/8/1 11:53
"""
from kuto import Page, AdrElem as Elem

"""
定位方式：优先选择rid
rid: 根据resourceId进行定位
text：根据text属性进行定位
cname：根据className属性进行定位
xpath：根据xpath进行定位
index：获取定位到的第index个元素
"""


class HomePage(Page):
    """APP首页"""
    adBtn = Elem(rid='com.qizhidao.clientapp:id/bottom_btn')
    myTab = Elem(xpath='//*[@resource-id="com.qizhidao.clientapp:id/ll'
                    'BottomTabs"]/android.widget.FrameLayout[4]')


class MyPage(Page):
    """我的页"""
    settingBtn = Elem(rid='com.qizhidao.clientapp:id/me_top_bar_setting_iv')


class SettingPage(Page):
    """设置页"""
    title = Elem(rid='com.qizhidao.clientapp:id/tv_actionbar_title')
    agreementText = Elem(rid='com.qizhidao.clientapp:id/agreement_tv_2')
